<?php

namespace Drupal\vitals;

use Drupal\update\UpdateManagerInterface;

/**
 * Class VitalsGetData.
 *
 * @package Drupal\vitals
 */
class Vitals {

  /**
   * The 403 HTTP Response Code.
   */
  const VITALS_403 = 403;

  /**
   * The 404 HTTP Response Code.
   */
  const VITALS_404 = 404;

  /**
   * Gets a generated authentication token.
   */
  public function generateToken() {
    return bin2hex(random_bytes(64));
  }

  /**
   * Gets site status.
   */
  public function getStatus() {
    $info = [
      'php_version' => $this->getPhpVersion(),
      'cms_version' => $this->getDrupalVersion(),
      'updates' => $this->getUpdates(),
    ];

    return $info;
  }

  /**
   * Gets the PHP version of the site.
   */
  public function getPhpVersion() {
    return sprintf(
      '%d.%d.%d',
      PHP_MAJOR_VERSION,
      PHP_MINOR_VERSION,
      PHP_RELEASE_VERSION
    );
  }

  /**
   * Gets the Drupal version of the site.
   */
  public function getDrupalVersion() {
    return \DRUPAL::VERSION;
  }

  /**
   * Gets version data for installed modules.
   */
  public function getExtensions() {
    $extensions = [];

    if ($available = update_get_available(TRUE)) {
      module_load_include('inc', 'update', 'update.compare');
      $extensions = update_calculate_project_data($available);
    }

    return $extensions;
  }

  /**
   * Gets version data for installed modules.
   */
  public function getUpdates() {
    $extensions = $this->getExtensions();

    $updates = [];
    $security_updates = [];

    foreach ($extensions as $extension) {
      $status = $extension['status'];
      $machine_name = $extension['name'];
      $recommended_version = $extension['recommended'];

      switch ($status) {
        case UpdateManagerInterface::NOT_SECURE:
        case UpdateManagerInterface::REVOKED:
        case UpdateManagerInterface::NOT_SUPPORTED:
          $security_updates[$machine_name] = $recommended_version;
          $updates[$machine_name] = $recommended_version;
          break;

        case UpdateManagerInterface::NOT_CURRENT:
          $updates[$machine_name] = $recommended_version;
          break;
      }
    }

    $update_data = [
      'pending_updates' => $updates,
      'security_updates' => $security_updates,
    ];

    return $update_data;
  }

}
