<?php

namespace Drupal\Tests\vitals\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Core\Url;
use Drupal\vitals\Vitals;

/**
 * Class VitalsTest.
 *
 * Functional tests for the Vitals module.
 *
 * @group Vitals
 */
class VitalsTest extends BrowserTestBase {

  /**
   * Modules to install.
   *
   * This array of modules will be enabled when the fixture Drupal site is
   * built. We leave it empty here because this is a skeleton test. A typical
   * test will enable basic modules like node and user.
   *
   * @var string[]
   */
  public static $modules = ['vitals'];

  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $user = $this->drupalCreateUser(['access content']);
    $this->drupalLogin($user);
  }

  /**
   * Tests happy path scenerio.
   */
  public function testVitalsSuccess() {
    $state = $this->container->get('state');
    $token = $state->get('vitals.token');
    $url = Url::fromRoute('vitals.content', ['token' => $token])->toString();

    $this->drupalGet($url);

    // Checking for the keys we know should exist.
    $this->assertResponse(200);
    $this->assertText('php_version');
    $this->assertText('cms_version');
    $this->assertText('security_updates');
    $this->assertText('pending_updates');
  }

  /**
   * Tests 404 configuration and response.
   */
  public function testVitals404() {
    $config = $this->container->get('config.factory')->getEditable('vitals.settings');
    $config->set('vitals_unauthorized_action', Vitals::VITALS_404)->save();

    $this->drupalGet('vitals/invalid-token');
    $this->assertResponse(404);
  }

  /**
   * Tests flood prevention (403)
   */
  public function testFloodPrevention403() {
    $config = $this->container->get('config.factory')->getEditable('vitals.settings');
    $config->set('vitals_unauthorized_action', Vitals::VITALS_403)->save();

    // Simulate and check several invalid requests.
    for ($i = 0; $i < 11; $i++) {
      $this->drupalGet('vitals/invalid-token');
      $this->assertResponse(403);
    }

    // Simulate a valid request and ensure it is rejected.
    $state = $this->container->get('state');
    $token = $state->get('vitals.token');
    $url = Url::fromRoute('vitals.content', ['token' => $token])->toString();

    $this->drupalGet($url);
    $this->assertResponse(403);
  }

  /**
   * Tests flood prevention (404)
   */
  public function testFloodPrevention404() {
    $config = $this->container->get('config.factory')->getEditable('vitals.settings');
    $config->set('vitals_unauthorized_action', Vitals::VITALS_404)->save();

    // Simulate and check several invalid requests.
    for ($i = 0; $i < 11; $i++) {
      $this->drupalGet('vitals/invalid-token');
      $this->assertResponse(404);
    }

    // Simulate a valid request and ensure it is rejected.
    $state = $this->container->get('state');
    $token = $state->get('vitals.token');
    $url = Url::fromRoute('vitals.content', ['token' => $token])->toString();

    $this->drupalGet($url);
    $this->assertResponse(404);
  }

  /**
   * Tests partial flood prevention.
   */
  public function testPartialFloodPrevention() {
    $config = $this->container->get('config.factory')->getEditable('vitals.settings');
    $config->set('vitals_unauthorized_action', Vitals::VITALS_404)->save();

    // Simulate and check several invalid requests.
    for ($i = 0; $i < 5; $i++) {
      $this->drupalGet('vitals/invalid-token');
      $this->assertResponse(404);
    }

    // Simulate a valid request and ensure it is rejected.
    $state = $this->container->get('state');
    $token = $state->get('vitals.token');
    $url = Url::fromRoute('vitals.content', ['token' => $token])->toString();

    $this->drupalGet($url);
    $this->assertResponse(200);
  }

}
